library persisted_bloc_stream;

import 'dart:convert';

import 'package:bloc_stream/bloc_stream.dart';
import 'package:dartz/dartz.dart';

import 'storage.dart';
export 'storage.dart';

abstract class PersistedBlocStream<State> extends BlocStream<State> {
  PersistedBlocStream(State initialValue) : super(initialValue);

  static Storage storage;

  static bool get _hasStorage {
    if (storage == null) {
      throw '''
        PersistedBlocStream.storage has not been assigned.
        Please ensure you have initialized and assigned a Storage instance.
        E.g.
        PersistedBlocStream.storage = await HiveStorage.build();
      ''';
    }

    return true;
  }

  String get storageKey => runtimeType.toString();

  Option<State> _readValue() {
    if (!_hasStorage) return none();
    return storage.read(storageKey).bind<State>(
        (json) => catching(() => fromJson(jsonDecode(json))).toOption());
  }

  @override
  State get initialValueOverride => _readValue().getOrElse(() => null);

  @override
  void willUpdateValue(State current, State next) {
    if (!_hasStorage) return;

    try {
      final json = toJson(next);
      if (json == null) return;
      storage.write(storageKey, jsonEncode(json));
    } catch (_) {}
  }

  State fromJson(dynamic json);
  dynamic toJson(State value);
}
